﻿-- locals, helperfunctions
local string_gsub, string_format = string.gsub, string.format
local table_sort, table_concat = table.sort, table.concat
local function Printmsg(...) print("|cFF33FF99Ailo|r:", ...) end

-- LibDataBroker
local ldb = LibStub:GetLibrary("LibDataBroker-1.1", true)
if not ldb then return end

Ailo = {}

AiloLDB = AiloLDB or {}
AiloDB = AiloDB or {}
AiloDB.minimapIcon = AiloDB.minimapIcon or {}

AiloLDB = ldb:NewDataObject("Ailo", {
	type = "data source",
	text = "Ailo",
	icon = "Interface\\Icons\\Achievement_Dungeon_UlduarRaid_Archway_01.png",
	OnClick = function(clickedframe, button)
		if button == "RightButton" then Ailo:PrintRaids() else Ailo:ManualPlayerUpdate() end
	end,
})

local LDBIcon = LibStub("LibDBIcon-1.0",true)
if LDBIcon then 
  LDBIcon:Register("Ailo", AiloLDB, AiloDB.minimapIcon)
end


local frame = CreateFrame("Frame")
local function onEventFunc(self, event, ...)
	if event == "UPDATE_INSTANCE_INFO" then
		self:UpdateRaidIDDB()
	end
end
frame:RegisterEvent("UPDATE_INSTANCE_INFO")
frame:SetScript("OnEvent", onEventFunc)

frame.InstanceNameFormatString = "%s%d%s"
function Ailo:ManualPlayerUpdate()
    local thisplayer, currentRaid = UnitName("player")
    if not AiloDB[thisplayer] then 
        AiloDB[thisplayer] = {}
    else 
        for currentRaid, _ in pairs(AiloDB[thisplayer]) do
            AiloDB[thisplayer][currentRaid] = nil
        end
    end
    frame:UpdateRaidIDDB()
end

function Ailo:PrintRaids()
    local currentPlayer, savedRaids, currentRaid, currentRaidExpire, numRaids, i = nil, {}
    local now = time()
    for currentPlayer, _ in pairs(AiloDB) do 
        numRaids = 0
        for currentRaid, currentRaidExpire in pairs(AiloDB[currentPlayer]) do
            if now > currentRaidExpire then
                AiloDB[currentPlayer][currentRaid] = nil
            else
                numRaids = numRaids+1 
                savedRaids[numRaids] = currentRaid
            end
        end
        if numRaids > 0 then 
            table_sort(savedRaids)
            Printmsg(currentPlayer..": "..table_concat(savedRaids, ", ")) 
            for numRaids,_ in ipairs(savedRaids) do 
                savedRaids[numRaids] = nil 
            end
       end
    end
end

function frame:UpdateRaidIDDB()
    local now, thisplayer, index = time(), UnitName("player")
    if not AiloDB[thisplayer] then AiloDB[thisplayer] = {} end
	for index=1,GetNumSavedInstances() do
	    --instanceName, instanceID, instanceReset, instanceDifficulty, locked, extended, instanceIDMostSig, isRaid, maxPlayers, difficultyName = GetSavedInstanceInfo(index)
       instanceName, _, instanceReset, instanceDifficulty, locked, _, _, isRaid, maxPlayers, _ = GetSavedInstanceInfo(index)
		if locked and isRaid then
		    AiloDB[thisplayer][string_format(self.InstanceNameFormatString, string_gsub(instanceName, "(%a)[%l%p]*[%s%-]*", "%1"), maxPlayers, instanceDifficulty > 2 and "hc" or "nhc")] = now+instanceReset
		end
	end
end

